# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: depot/build/v1/build.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x64\x65pot/build/v1/build.proto\x12\x0e\x64\x65pot.build.v1\"(\n\x12\x43reateBuildRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"<\n\x13\x43reateBuildResponse\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12\x13\n\x0b\x62uild_token\x18\x02 \x01(\t\"\xe1\x01\n\x12\x46inishBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12\x42\n\x07success\x18\x02 \x01(\x0b\x32/.depot.build.v1.FinishBuildRequest.BuildSuccessH\x00\x12>\n\x05\x65rror\x18\x03 \x01(\x0b\x32-.depot.build.v1.FinishBuildRequest.BuildErrorH\x00\x1a\x0e\n\x0c\x42uildSuccess\x1a\x1b\n\nBuildError\x12\r\n\x05\x65rror\x18\x01 \x01(\tB\x08\n\x06result\"\x15\n\x13\x46inishBuildResponse2\xbe\x01\n\x0c\x42uildService\x12V\n\x0b\x43reateBuild\x12\".depot.build.v1.CreateBuildRequest\x1a#.depot.build.v1.CreateBuildResponse\x12V\n\x0b\x46inishBuild\x12\".depot.build.v1.FinishBuildRequest\x1a#.depot.build.v1.FinishBuildResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'depot.build.v1.build_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CREATEBUILDREQUEST']._serialized_start=46
  _globals['_CREATEBUILDREQUEST']._serialized_end=86
  _globals['_CREATEBUILDRESPONSE']._serialized_start=88
  _globals['_CREATEBUILDRESPONSE']._serialized_end=148
  _globals['_FINISHBUILDREQUEST']._serialized_start=151
  _globals['_FINISHBUILDREQUEST']._serialized_end=376
  _globals['_FINISHBUILDREQUEST_BUILDSUCCESS']._serialized_start=323
  _globals['_FINISHBUILDREQUEST_BUILDSUCCESS']._serialized_end=337
  _globals['_FINISHBUILDREQUEST_BUILDERROR']._serialized_start=339
  _globals['_FINISHBUILDREQUEST_BUILDERROR']._serialized_end=366
  _globals['_FINISHBUILDRESPONSE']._serialized_start=378
  _globals['_FINISHBUILDRESPONSE']._serialized_end=399
  _globals['_BUILDSERVICE']._serialized_start=402
  _globals['_BUILDSERVICE']._serialized_end=592
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from typing import List, Optional, Union

import grpc

import depot_client.api.depot.core.v1.project_pb2 as project_pb2
import depot_client.api.depot.core.v1.project_pb2_grpc as project_pb2_grpc
from depot_client.api.depot.core.v1.project_pb2 import (
    Project as ProjectProto,
)
from depot_client.api.depot.core.v1.project_pb2 import (
    TrustPolicy as TrustPolicyProto,
)


@dataclass
class ProjectInfo:
    project_id: str
    organization_id: str
    name: str
    region_id: str
    created_at: str
    cache_policy: Optional[dict] = None
    hardware: Optional[str] = None

    @classmethod
    def from_proto(cls, proto: ProjectProto) -> "ProjectInfo":
        return cls(
            project_id=proto.project_id,
            organization_id=proto.organization_id,
            name=proto.name,
            region_id=proto.region_id,
            created_at=proto.created_at.ToDatetime().isoformat(),
            cache_policy=proto.cache_policy if proto.HasField("cache_policy") else None,
            hardware=proto.hardware if proto.hardware != 0 else None,
        )


@dataclass
class TokenInfo:
    token_id: str
    description: str


@dataclass
class TokenCreationInfo:
    token_id: str
    secret: str


@dataclass
class TrustPolicyGitHub:
    repository_owner: str
    repository: str


@dataclass
class TrustPolicyCircleCI:
    organization_uuid: str
    project_uuid: str


@dataclass
class TrustPolicyBuildkite:
    organization_slug: str
    pipeline_slug: str


@dataclass
class TrustPolicy:
    trust_policy_id: str
    github: Optional[TrustPolicyGitHub] = None
    circleci: Optional[TrustPolicyCircleCI] = None
    buildkite: Optional[TrustPolicyBuildkite] = None

    @classmethod
    def from_proto(cls, proto: TrustPolicyProto) -> "TrustPolicy":
        kwargs = {"trust_policy_id": proto.trust_policy_id}

        if proto.HasField("github"):
            kwargs["github"] = TrustPolicyGitHub(
                repository_owner=proto.github.repository_owner,
                repository=proto.github.repository,
            )
        elif proto.HasField("circleci"):
            kwargs["circleci"] = TrustPolicyCircleCI(
                organization_uuid=proto.circleci.organization_uuid,
                project_uuid=proto.circleci.project_uuid,
            )
        elif proto.HasField("buildkite"):
            kwargs["buildkite"] = TrustPolicyBuildkite(
                organization_slug=proto.buildkite.organization_slug,
                pipeline_slug=proto.buildkite.pipeline_slug,
            )

        return cls(**kwargs)


class ProjectService:
    def __init__(self, channel: grpc.Channel):
        self.stub = project_pb2_grpc.ProjectServiceStub(channel)

    def list_projects(self) -> List[ProjectInfo]:
        request = project_pb2.ListProjectsRequest()
        response = self.stub.ListProjects(request)
        return [ProjectInfo.from_proto(project) for project in response.projects]

    def create_project(self, name: str, region_id: str) -> ProjectInfo:
        request = project_pb2.CreateProjectRequest(name=name, region_id=region_id)
        response = self.stub.CreateProject(request)
        return ProjectInfo.from_proto(response.project)

    def update_project(self, project_id: str, name: str, region_id: str) -> ProjectInfo:
        request = project_pb2.UpdateProjectRequest(
            project_id=project_id, name=name, region_id=region_id
        )
        response = self.stub.UpdateProject(request)
        return ProjectInfo.from_proto(response.project)

    def delete_project(self, project_id: str) -> None:
        request = project_pb2.DeleteProjectRequest(project_id=project_id)
        self.stub.DeleteProject(request)

    def reset_project(self, project_id: str) -> None:
        request = project_pb2.ResetProjectRequest(project_id=project_id)
        self.stub.ResetProject(request)

    def list_trust_policies(self, project_id: str) -> List[TrustPolicy]:
        request = project_pb2.ListTrustPoliciesRequest(project_id=project_id)
        response = self.stub.ListTrustPolicies(request)
        return [TrustPolicy.from_proto(policy) for policy in response.trust_policies]

    def add_trust_policy(
        self,
        project_id: str,
        provider: Union[dict, None] = None,
        buildkite: Optional[dict] = None,
        circleci: Optional[dict] = None,
        github: Optional[dict] = None,
    ) -> TrustPolicy:
        if provider:
            # Handle legacy provider dict
            if provider.get("type") == "github":
                github = provider
            elif provider.get("type") == "circleci":
                circleci = provider
            elif provider.get("type") == "buildkite":
                buildkite = provider

        request = project_pb2.AddTrustPolicyRequest(project_id=project_id)

        if github:
            request.github.repository_owner = github["repository_owner"]
            request.github.repository = github["repository"]
        elif circleci:
            request.circleci.organization_uuid = circleci["organization_uuid"]
            request.circleci.project_uuid = circleci["project_uuid"]
        elif buildkite:
            request.buildkite.organization_slug = buildkite["organization_slug"]
            request.buildkite.pipeline_slug = buildkite["pipeline_slug"]

        response = self.stub.AddTrustPolicy(request)
        return TrustPolicy.from_proto(response.trust_policy)

    def remove_trust_policy(self, project_id: str, trust_policy_id: str) -> None:
        request = project_pb2.RemoveTrustPolicyRequest(
            project_id=project_id,
            trust_policy_id=trust_policy_id,
        )
        self.stub.RemoveTrustPolicy(request)

    def list_tokens(self, project_id: str) -> List[TokenInfo]:
        request = project_pb2.ListTokensRequest(project_id=project_id)
        response = self.stub.ListTokens(request)
        return [
            TokenInfo(token_id=token.token_id, description=token.description)
            for token in response.tokens
        ]

    def create_token(self, project_id: str, description: str) -> TokenCreationInfo:
        request = project_pb2.CreateTokenRequest(
            project_id=project_id, description=description
        )
        response = self.stub.CreateToken(request)
        return TokenCreationInfo(token_id=response.token_id, secret=response.secret)

    def update_token(self, token_id: str, description: str) -> None:
        request = project_pb2.UpdateTokenRequest(
            token_id=token_id, description=description
        )
        self.stub.UpdateToken(request)

    def delete_token(self, token_id: str) -> None:
        request = project_pb2.DeleteTokenRequest(token_id=token_id)
        self.stub.DeleteToken(request)


class AsyncProjectService:
    def __init__(self, channel: grpc.Channel):
        self.stub = project_pb2_grpc.ProjectServiceStub(channel)

    async def list_projects(self) -> List[ProjectInfo]:
        request = project_pb2.ListProjectsRequest()
        response = await self.stub.ListProjects(request)
        return [ProjectInfo.from_proto(project) for project in response.projects]

    async def create_project(self, name: str, region_id: str) -> ProjectInfo:
        request = project_pb2.CreateProjectRequest(name=name, region_id=region_id)
        response = await self.stub.CreateProject(request)
        return ProjectInfo.from_proto(response.project)

    async def update_project(
        self, project_id: str, name: str, region_id: str
    ) -> ProjectInfo:
        request = project_pb2.UpdateProjectRequest(
            project_id=project_id, name=name, region_id=region_id
        )
        response = await self.stub.UpdateProject(request)
        return ProjectInfo.from_proto(response.project)

    async def delete_project(self, project_id: str) -> None:
        request = project_pb2.DeleteProjectRequest(project_id=project_id)
        await self.stub.DeleteProject(request)

    async def reset_project(self, project_id: str) -> None:
        request = project_pb2.ResetProjectRequest(project_id=project_id)
        await self.stub.ResetProject(request)

    async def list_trust_policies(self, project_id: str) -> List[TrustPolicy]:
        request = project_pb2.ListTrustPoliciesRequest(project_id=project_id)
        response = await self.stub.ListTrustPolicies(request)
        return [TrustPolicy.from_proto(policy) for policy in response.trust_policies]

    async def add_trust_policy(
        self,
        project_id: str,
        provider: Union[dict, None] = None,
        buildkite: Optional[dict] = None,
        circleci: Optional[dict] = None,
        github: Optional[dict] = None,
    ) -> TrustPolicy:
        if provider:
            # Handle legacy provider dict
            if provider.get("type") == "github":
                github = provider
            elif provider.get("type") == "circleci":
                circleci = provider
            elif provider.get("type") == "buildkite":
                buildkite = provider

        request = project_pb2.AddTrustPolicyRequest(project_id=project_id)

        if github:
            request.github.repository_owner = github["repository_owner"]
            request.github.repository = github["repository"]
        elif circleci:
            request.circleci.organization_uuid = circleci["organization_uuid"]
            request.circleci.project_uuid = circleci["project_uuid"]
        elif buildkite:
            request.buildkite.organization_slug = buildkite["organization_slug"]
            request.buildkite.pipeline_slug = buildkite["pipeline_slug"]

        response = await self.stub.AddTrustPolicy(request)
        return TrustPolicy.from_proto(response.trust_policy)

    async def remove_trust_policy(self, project_id: str, trust_policy_id: str) -> None:
        request = project_pb2.RemoveTrustPolicyRequest(
            project_id=project_id,
            trust_policy_id=trust_policy_id,
        )
        await self.stub.RemoveTrustPolicy(request)

    async def list_tokens(self, project_id: str) -> List[TokenInfo]:
        request = project_pb2.ListTokensRequest(project_id=project_id)
        response = await self.stub.ListTokens(request)
        return [
            TokenInfo(token_id=token.token_id, description=token.description)
            for token in response.tokens
        ]

    async def create_token(
        self, project_id: str, description: str
    ) -> TokenCreationInfo:
        request = project_pb2.CreateTokenRequest(
            project_id=project_id, description=description
        )
        response = await self.stub.CreateToken(request)
        return TokenCreationInfo(token_id=response.token_id, secret=response.secret)

    async def update_token(self, token_id: str, description: str) -> None:
        request = project_pb2.UpdateTokenRequest(
            token_id=token_id, description=description
        )
        await self.stub.UpdateToken(request)

    async def delete_token(self, token_id: str) -> None:
        request = project_pb2.DeleteTokenRequest(token_id=token_id)
        await self.stub.DeleteToken(request)



import asyncio
import json
import logging
import os
import threading
import time
from contextlib import asynccontextmanager, contextmanager
from dataclasses import dataclass
from datetime import datetime
from typing import AsyncIterator, Iterator, List, Optional, Union

import grpc
import grpc.aio
from google.protobuf.timestamp_pb2 import Timestamp

from depot_client.build import AsyncBuildService, BuildService
from depot_client.buildkit import AsyncBuildKitService, BuildKitService, EndpointInfo
from depot_client.core_build import AsyncCoreBuildService, BuildInfo, CoreBuildService
from depot_client.project import (
    AsyncProjectService,
    ProjectInfo,
    ProjectService,
    TokenCreationInfo,
    TokenInfo,
    TrustPolicy,
)

DEPOT_GRPC_HOST = "api.depot.dev"
DEPOT_GRPC_PORT = 443

SERVICE_CONFIG_JSON = json.dumps(
    {
        "methodConfig": [
            {
                "name": [{}],
                "retryPolicy": {
                    "maxAttempts": 5,
                    "initialBackoff": "0.1s",
                    "maxBackoff": "5s",
                    "backoffMultiplier": 2,
                    "retryableStatusCodes": ["UNAVAILABLE"],
                },
            }
        ]
    }
)

CHANNEL_OPTIONS = [
    ("grpc.enable_retries", 1),
    ("grpc.service_config", SERVICE_CONFIG_JSON),
]

REPORT_HEALTH_INTERVAL = 60
REPORT_HEALTH_SLEEP_INTERVAL = 0.2
REPORT_HEALTH_CANCEL_TIMEOUT = 1

logger = logging.getLogger(__name__)


@dataclass
class Endpoint(EndpointInfo):
    build_id: str
    platform: str
    buildkit: BuildKitService

    def _report_health(self):
        last_report = 0.0
        while not self._stop_health.is_set():
            if time.time() - last_report > REPORT_HEALTH_INTERVAL:
                self.buildkit.report_health(self.build_id, self.platform)
                last_report = time.time()
            time.sleep(REPORT_HEALTH_SLEEP_INTERVAL)

    def __enter__(self):
        self._health_thread = threading.Thread(target=self._report_health, daemon=True)
        self._stop_health = threading.Event()
        self._health_thread.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._stop_health.set()
        self._health_thread.join(timeout=REPORT_HEALTH_CANCEL_TIMEOUT)
        self.close()

    def close(self):
        self.buildkit.release_endpoint(self.build_id, self.platform)


@dataclass
class AsyncEndpoint(EndpointInfo):
    build_id: str
    platform: str
    buildkit: AsyncBuildKitService

    async def _report_health(self):
        loop = asyncio.get_running_loop()
        last_report = 0.0
        while True:
            if loop.time() - last_report > REPORT_HEALTH_INTERVAL:
                await self.buildkit.report_health(self.build_id, self.platform)
                last_report = loop.time()
            await asyncio.sleep(REPORT_HEALTH_SLEEP_INTERVAL)

    async def __aenter__(self):
        self._health_task = asyncio.create_task(self._report_health())
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        self._health_task.cancel()
        try:
            await asyncio.wait_for(
                self._health_task, timeout=REPORT_HEALTH_CANCEL_TIMEOUT
            )
        except (asyncio.CancelledError, asyncio.TimeoutError):
            pass
        await self.close()

    async def close(self):
        await self.buildkit.release_endpoint(self.build_id, self.platform)


class Build:
    def __init__(
        self,
        build_service,
        build_id: str,
        build_token: str,
        buildkit_host: str = DEPOT_GRPC_HOST,
        buildkit_port: int = DEPOT_GRPC_PORT,
    ):
        self.build_service = build_service
        self.build_id = build_id
        self.build_token = build_token
        self.buildkit = BuildKitService(
            buildkit_host,
            buildkit_port,
            build_token,
        )

    def close(self):
        self.buildkit.close()
        self.build_service.finish_build(self.build_id)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def get_endpoint(self, platform: Optional[str] = None) -> Endpoint:
        endpoint = self.buildkit.get_endpoint(self.build_id, platform=platform)
        return Endpoint(
            endpoint=endpoint.endpoint,
            server_name=endpoint.server_name,
            cert=endpoint.cert,
            key=endpoint.key,
            ca_cert=endpoint.ca_cert,
            build_id=self.build_id,
            platform=platform,
            buildkit=self.buildkit,
        )


class AsyncBuild:
    def __init__(
        self,
        build_service,
        build_id: str,
        build_token: str,
        buildkit_host: str = DEPOT_GRPC_HOST,
        buildkit_port: int = DEPOT_GRPC_PORT,
    ):
        self.build_service = build_service
        self.build_id = build_id
        self.build_token = build_token
        self.buildkit = AsyncBuildKitService(
            buildkit_host,
            buildkit_port,
            build_token,
        )

    async def close(self):
        await self.buildkit.close()
        await self.build_service.finish_build(self.build_id)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close()

    async def get_endpoint(self, platform: Optional[str] = None) -> AsyncEndpoint:
        endpoint = await self.buildkit.get_endpoint(self.build_id, platform=platform)
        return AsyncEndpoint(
            endpoint=endpoint.endpoint,
            server_name=endpoint.server_name,
            cert=endpoint.cert,
            key=endpoint.key,
            ca_cert=endpoint.ca_cert,
            build_id=self.build_id,
            platform=platform,
            buildkit=self.buildkit,
        )


class BaseClient:
    def _create_channel_credentials(
        self, token: Optional[str] = None
    ) -> grpc.ChannelCredentials:
        channel_creds = grpc.ssl_channel_credentials()
        call_creds = grpc.access_token_call_credentials(
            token or os.getenv("DEPOT_API_TOKEN")
        )
        return grpc.composite_channel_credentials(channel_creds, call_creds)

    def _proto_to_datetime(self, timestamp: Timestamp) -> datetime:
        return datetime.fromtimestamp(timestamp.seconds + timestamp.nanos / 1e9)


class Client(BaseClient):
    def __init__(
        self,
        host: str = DEPOT_GRPC_HOST,
        port: int = DEPOT_GRPC_PORT,
        token: Optional[str] = None,
    ):
        credentials = self._create_channel_credentials(token)
        self.host = host
        self.port = port
        self.channel = grpc.secure_channel(
            f"{host}:{port}",
            credentials,
            options=CHANNEL_OPTIONS,
        )
        self.build = BuildService(self.channel)
        self.core_build = CoreBuildService(self.channel)
        self.project = ProjectService(self.channel)

    def close(self):
        self.channel.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def list_projects(self) -> List[ProjectInfo]:
        return self.project.list_projects()

    def create_build(self, project_id: str) -> Build:
        build_id, build_token = self.build.create_build(project_id)
        return Build(self.build, build_id=build_id, build_token=build_token)

    def finish_build(self, build_id: str, error: Optional[str] = None) -> None:
        return self.build.finish_build(build_id, error=error)

    def share_build(self, build_id: str) -> str:
        return self.core_build.share_build(build_id)

    def stop_sharing_build(self, build_id: str) -> None:
        return self.core_build.stop_sharing_build(build_id)

    def get_build(self, build_id: str) -> BuildInfo:
        return self.core_build.get_build(build_id)

    def list_builds(
        self,
        project_id: str,
    ) -> List[BuildInfo]:
        return self.core_build.list_builds(project_id)

    @contextmanager
    def create_endpoint(
        self, project_id: str, platform: Optional[str] = None
    ) -> Iterator[Endpoint]:
        with self.create_build(project_id) as build:
            with build.get_endpoint(platform=platform) as endpoint:
                yield endpoint

    def get_project(self, project_id: str) -> ProjectInfo:
        return self.project.get_project(project_id)

    def create_project(
        self,
        name: str,
        organization_id: Optional[str] = None,
        region_id: Optional[str] = None,
        cache_policy: Optional[dict] = None,
        hardware: Optional[str] = None,
    ) -> ProjectInfo:
        return self.project.create_project(
            name=name,
            organization_id=organization_id,
            region_id=region_id,
            cache_policy=cache_policy,
            hardware=hardware,
        )

    def update_project(
        self,
        project_id: str,
        name: Optional[str] = None,
        region_id: Optional[str] = None,
        cache_policy: Optional[dict] = None,
        hardware: Optional[str] = None,
    ) -> ProjectInfo:
        return self.project.update_project(
            project_id=project_id,
            name=name,
            region_id=region_id,
            cache_policy=cache_policy,
            hardware=hardware,
        )

    def delete_project(self, project_id: str) -> None:
        return self.project.delete_project(project_id)

    def reset_project(self, project_id: str) -> None:
        return self.project.reset_project(project_id)

    def list_trust_policies(self, project_id: str) -> List[TrustPolicy]:
        return self.project.list_trust_policies(project_id)

    def add_trust_policy(
        self,
        project_id: str,
        provider: Union[dict, None] = None,
        buildkite: Optional[dict] = None,
        circleci: Optional[dict] = None,
        github: Optional[dict] = None,
    ) -> TrustPolicy:
        return self.project.add_trust_policy(
            project_id=project_id,
            provider=provider,
            buildkite=buildkite,
            circleci=circleci,
            github=github,
        )

    def remove_trust_policy(self, project_id: str, trust_policy_id: str) -> None:
        return self.project.remove_trust_policy(project_id, trust_policy_id)

    def list_tokens(self, project_id: str) -> List[TokenInfo]:
        return self.project.list_tokens(project_id)

    def create_token(self, project_id: str, description: str) -> TokenCreationInfo:
        return self.project.create_token(project_id, description)

    def update_token(self, token_id: str, description: str) -> None:
        return self.project.update_token(token_id, description)

    def delete_token(self, token_id: str) -> None:
        return self.project.delete_token(token_id)


class AsyncClient(BaseClient):
    def __init__(
        self,
        host: str = DEPOT_GRPC_HOST,
        port: int = DEPOT_GRPC_PORT,
        token: Optional[str] = None,
    ):
        credentials = self._create_channel_credentials(token)
        self.channel = grpc.aio.secure_channel(
            f"{host}:{port}",
            credentials,
            options=CHANNEL_OPTIONS,
        )
        self.build = AsyncBuildService(self.channel)
        self.core_build = AsyncCoreBuildService(self.channel)
        self.project = AsyncProjectService(self.channel)

    async def close(self):
        await self.channel.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close()

    async def list_projects(self) -> List[ProjectInfo]:
        return await self.project.list_projects()

    async def create_build(self, project_id: str) -> AsyncBuild:
        build_id, build_token = await self.build.create_build(project_id)
        return AsyncBuild(self.build, build_id=build_id, build_token=build_token)

    async def finish_build(self, build_id: str, error: Optional[str] = None) -> None:
        return await self.build.finish_build(build_id, error=error)

    async def share_build(self, build_id: str) -> str:
        return await self.core_build.share_build(build_id)

    async def stop_sharing_build(self, build_id: str) -> None:
        return await self.core_build.stop_sharing_build(build_id)

    async def get_build(self, build_id: str) -> BuildInfo:
        return await self.core_build.get_build(build_id)

    async def list_builds(
        self,
        project_id: str,
    ) -> List[BuildInfo]:
        return await self.core_build.list_builds(project_id)

    @asynccontextmanager
    async def create_endpoint(
        self, project_id: str, platform: Optional[str] = None
    ) -> AsyncIterator[AsyncEndpoint]:
        async with await self.create_build(project_id) as build:
            async with await build.get_endpoint(platform=platform) as endpoint:
                yield endpoint

    async def get_project(self, project_id: str) -> ProjectInfo:
        return await self.project.get_project(project_id)

    async def create_project(
        self,
        name: str,
        organization_id: Optional[str] = None,
        region_id: Optional[str] = None,
        cache_policy: Optional[dict] = None,
        hardware: Optional[str] = None,
    ) -> ProjectInfo:
        return await self.project.create_project(
            name=name,
            organization_id=organization_id,
            region_id=region_id,
            cache_policy=cache_policy,
            hardware=hardware,
        )

    async def update_project(
        self,
        project_id: str,
        name: Optional[str] = None,
        region_id: Optional[str] = None,
        cache_policy: Optional[dict] = None,
        hardware: Optional[str] = None,
    ) -> ProjectInfo:
        return await self.project.update_project(
            project_id=project_id,
            name=name,
            region_id=region_id,
            cache_policy=cache_policy,
            hardware=hardware,
        )

    async def delete_project(self, project_id: str) -> None:
        return await self.project.delete_project(project_id)

    async def reset_project(self, project_id: str) -> None:
        return await self.project.reset_project(project_id)

    async def list_trust_policies(self, project_id: str) -> List[TrustPolicy]:
        return await self.project.list_trust_policies(project_id)

    async def add_trust_policy(
        self,
        project_id: str,
        provider: Union[dict, None] = None,
        buildkite: Optional[dict] = None,
        circleci: Optional[dict] = None,
        github: Optional[dict] = None,
    ) -> TrustPolicy:
        return await self.project.add_trust_policy(
            project_id=project_id,
            provider=provider,
            buildkite=buildkite,
            circleci=circleci,
            github=github,
        )

    async def remove_trust_policy(self, project_id: str, trust_policy_id: str) -> None:
        return await self.project.remove_trust_policy(project_id, trust_policy_id)

    async def list_tokens(self, project_id: str) -> List[TokenInfo]:
        return await self.project.list_tokens(project_id)

    async def create_token(
        self, project_id: str, description: str
    ) -> TokenCreationInfo:
        return await self.project.create_token(project_id, description)

    async def update_token(self, token_id: str, description: str) -> None:
        return await self.project.update_token(token_id, description)

    async def delete_token(self, token_id: str) -> None:
        return await self.project.delete_token(token_id)


def _main():
    with Client() as client:
        print(client.list_projects())
        project_id = "749dxclhrj"
        client.list_builds(project_id)
        with client.create_endpoint(project_id) as endpoint:
            print(repr(endpoint))
            assert isinstance(endpoint.cert, str)
            assert isinstance(endpoint.key, str)
            assert isinstance(endpoint.ca_cert, str)


async def _async_main():
    async with AsyncClient() as client:
        print(await client.list_projects())
        project_id = "749dxclhrj"
        await client.list_builds(project_id)
        async with await client.create_build(project_id) as build:
            async with await build.get_endpoint() as endpoint:
                print(repr(endpoint))
                assert isinstance(endpoint.cert, str)
                assert isinstance(endpoint.key, str)
                assert isinstance(endpoint.ca_cert, str)


if __name__ == "__main__":
    _main()
    asyncio.run(_async_main())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: depot/core/v1/build.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x64\x65pot/core/v1/build.proto\x12\rdepot.core.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"%\n\x11ShareBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\"\'\n\x12ShareBuildResponse\x12\x11\n\tshare_url\x18\x01 \x01(\t\"+\n\x17StopSharingBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\"\x1a\n\x18StopSharingBuildResponse\"u\n\x11ListBuildsRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x16\n\tpage_size\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x17\n\npage_token\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x0c\n\n_page_sizeB\r\n\x0b_page_token\"S\n\x12ListBuildsResponse\x12$\n\x06\x62uilds\x18\x01 \x03(\x0b\x32\x14.depot.core.v1.Build\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"#\n\x0fGetBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\"7\n\x10GetBuildResponse\x12#\n\x05\x62uild\x18\x01 \x01(\x0b\x32\x14.depot.core.v1.Build\"\xdb\x04\n\x05\x42uild\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12+\n\x06status\x18\x02 \x01(\x0e\x32\x1b.depot.core.v1.Build.Status\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\nstarted_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x34\n\x0b\x66inished_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01\x88\x01\x01\x12#\n\x16\x62uild_duration_seconds\x18\x06 \x01(\x05H\x02\x88\x01\x01\x12#\n\x16saved_duration_seconds\x18\x07 \x01(\x05H\x03\x88\x01\x01\x12\x19\n\x0c\x63\x61\x63hed_steps\x18\x08 \x01(\x05H\x04\x88\x01\x01\x12\x18\n\x0btotal_steps\x18\t \x01(\x05H\x05\x88\x01\x01\"\x82\x01\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x12\n\x0eSTATUS_RUNNING\x10\x01\x12\x11\n\rSTATUS_FAILED\x10\x02\x12\x12\n\x0eSTATUS_SUCCESS\x10\x03\x12\x10\n\x0cSTATUS_ERROR\x10\x04\x12\x13\n\x0fSTATUS_CANCELED\x10\x05\x42\r\n\x0b_started_atB\x0e\n\x0c_finished_atB\x19\n\x17_build_duration_secondsB\x19\n\x17_saved_duration_secondsB\x0f\n\r_cached_stepsB\x0e\n\x0c_total_steps2\xe6\x02\n\x0c\x42uildService\x12Q\n\nShareBuild\x12 .depot.core.v1.ShareBuildRequest\x1a!.depot.core.v1.ShareBuildResponse\x12\x63\n\x10StopSharingBuild\x12&.depot.core.v1.StopSharingBuildRequest\x1a\'.depot.core.v1.StopSharingBuildResponse\x12Q\n\nListBuilds\x12 .depot.core.v1.ListBuildsRequest\x1a!.depot.core.v1.ListBuildsResponse\x12K\n\x08GetBuild\x12\x1e.depot.core.v1.GetBuildRequest\x1a\x1f.depot.core.v1.GetBuildResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'depot.core.v1.build_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SHAREBUILDREQUEST']._serialized_start=77
  _globals['_SHAREBUILDREQUEST']._serialized_end=114
  _globals['_SHAREBUILDRESPONSE']._serialized_start=116
  _globals['_SHAREBUILDRESPONSE']._serialized_end=155
  _globals['_STOPSHARINGBUILDREQUEST']._serialized_start=157
  _globals['_STOPSHARINGBUILDREQUEST']._serialized_end=200
  _globals['_STOPSHARINGBUILDRESPONSE']._serialized_start=202
  _globals['_STOPSHARINGBUILDRESPONSE']._serialized_end=228
  _globals['_LISTBUILDSREQUEST']._serialized_start=230
  _globals['_LISTBUILDSREQUEST']._serialized_end=347
  _globals['_LISTBUILDSRESPONSE']._serialized_start=349
  _globals['_LISTBUILDSRESPONSE']._serialized_end=432
  _globals['_GETBUILDREQUEST']._serialized_start=434
  _globals['_GETBUILDREQUEST']._serialized_end=469
  _globals['_GETBUILDRESPONSE']._serialized_start=471
  _globals['_GETBUILDRESPONSE']._serialized_end=526
  _globals['_BUILD']._serialized_start=529
  _globals['_BUILD']._serialized_end=1132
  _globals['_BUILD_STATUS']._serialized_start=884
  _globals['_BUILD_STATUS']._serialized_end=1014
  _globals['_BUILDSERVICE']._serialized_start=1135
  _globals['_BUILDSERVICE']._serialized_end=1493
# @@protoc_insertion_point(module_scope)



import json
from dataclasses import dataclass
from typing import Optional

import grpc

from depot_client.api.depot.buildkit.v1.buildkit_pb2 import (
    GetEndpointRequest,
    Platform,
    ReleaseEndpointRequest,
    ReportHealthRequest,
)
from depot_client.api.depot.buildkit.v1.buildkit_pb2_grpc import BuildKitServiceStub

BUILDKIT_SERVICE_CONFIG_JSON = json.dumps(
    {
        "methodConfig": [
            {
                "name": [{}],
                "retryPolicy": {
                    "maxAttempts": 5,
                    "initialBackoff": "0.1s",
                    "maxBackoff": "5s",
                    "backoffMultiplier": 2,
                    "retryableStatusCodes": [
                        "UNAVAILABLE",
                        # could be a race condition where it doesn't see the build yet
                        "NOT_FOUND",
                    ],
                },
            }
        ]
    }
)

BUILDKIT_CHANNEL_OPTIONS = [
    ("grpc.enable_retries", 1),
    ("grpc.service_config", BUILDKIT_SERVICE_CONFIG_JSON),
]

DEFAULT_PLATFORM = "amd64"

PLATFORM_MAP = {
    "amd64": "PLATFORM_AMD64",
    "arm64": "PLATFORM_ARM64",
}


def get_platform(platform: Optional[str] = None) -> str:
    return PLATFORM_MAP[platform or DEFAULT_PLATFORM]


@dataclass
class EndpointInfo:
    endpoint: str
    server_name: str
    cert: str
    key: str
    ca_cert: str


class BuildKitService:
    def __init__(self, host: str, port: int, token: str):
        creds = grpc.composite_channel_credentials(
            grpc.ssl_channel_credentials(),
            grpc.access_token_call_credentials(token),
        )
        self.channel = grpc.secure_channel(
            f"{host}:{port}",
            creds,
            options=BUILDKIT_CHANNEL_OPTIONS,
        )
        self.stub = BuildKitServiceStub(self.channel)

    def close(self):
        self.channel.close()

    def get_endpoint(self, build_id: str, platform: Optional[str] = None):
        request = GetEndpointRequest(
            build_id=build_id,
            platform=Platform.Value(get_platform(platform)),
        )
        for response in self.stub.GetEndpoint(request):
            if response.HasField("pending"):
                continue
            elif response.HasField("active"):
                print(dir(response.active.cert))
                return EndpointInfo(
                    endpoint=response.active.endpoint,
                    server_name=response.active.server_name,
                    cert=response.active.cert.cert.cert,
                    key=response.active.cert.key.key,
                    ca_cert=response.active.ca_cert.cert,
                )
            else:
                raise ValueError("Unknown response type: {response}")

    def report_health(self, build_id: str, platform: Optional[str] = None) -> None:
        request = ReportHealthRequest(
            build_id=build_id,
            platform=Platform.Value(get_platform(platform)),
        )

        def request_generator():
            yield request

        self.stub.ReportHealth(request_generator())

    def release_endpoint(self, build_id: str, platform: Optional[str] = None) -> None:
        request = ReleaseEndpointRequest(
            build_id=build_id,
            platform=Platform.Value(get_platform(platform)),
        )
        self.stub.ReleaseEndpoint(request)


class AsyncBuildKitService:
    def __init__(self, host: str, port: int, token: str):
        creds = grpc.composite_channel_credentials(
            grpc.ssl_channel_credentials(),
            grpc.access_token_call_credentials(token),
        )
        self.channel = grpc.aio.secure_channel(
            f"{host}:{port}",
            creds,
            options=BUILDKIT_CHANNEL_OPTIONS,
        )
        self.stub = BuildKitServiceStub(self.channel)

    async def close(self):
        await self.channel.close()

    async def get_endpoint(self, build_id: str, platform: Optional[str] = None):
        request = GetEndpointRequest(
            build_id=build_id,
            platform=Platform.Value(get_platform(platform)),
        )
        async for response in self.stub.GetEndpoint(request):
            if response.HasField("pending"):
                continue
            elif response.HasField("active"):
                print(dir(response.active.cert))
                return EndpointInfo(
                    endpoint=response.active.endpoint,
                    server_name=response.active.server_name,
                    cert=response.active.cert.cert.cert,
                    key=response.active.cert.key.key,
                    ca_cert=response.active.ca_cert.cert,
                )
            else:
                raise ValueError("Unknown response type: {response}")

    async def report_health(
        self, build_id: str, platform: Optional[str] = None
    ) -> None:
        request = ReportHealthRequest(
            build_id=build_id,
            platform=Platform.Value(get_platform(platform)),
        )

        def request_generator():
            yield request

        await self.stub.ReportHealth(request_generator())

    async def release_endpoint(
        self, build_id: str, platform: Optional[str] = None
    ) -> None:
        request = ReleaseEndpointRequest(
            build_id=build_id,
            platform=Platform.Value(get_platform(platform)),
        )
        await self.stub.ReleaseEndpoint(request)
